
#include <iostream>
using namespace std;

#include <lfc/Chain.h>
#include <lfc/Exception.h>
#include <lfc/GetLongOpt.h>
#include <lfc/File.h>
#include <lfc/Timer.h>
#include <lfc/Tokenizer.h>
#include <lfc/Net.h>

#include <cego/CegoModule.h>
#include <cego/CegoAdmNet.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#ifdef __cplusplus
}
#endif

class CegoCtlDriver {
public:
    
    CegoCtlDriver()  {
	
	Chain logFile("cegoctl.log");
	Chain logMode("notice");
	
	_pAdmNet=new CegoAdmNet(logFile, logMode);
		
    }

    ~CegoCtlDriver() { 
    } 

    void connect(char *server, int port, char* user, char* pwd) {

	_pAdmNet->connect(server, port, user, pwd);

    }

    char* getTSInfo(int i) {
	if ( i < _tsInfoList.Size() )
	    return (char*) _tsInfoList[i];
	else
	    return "";
    }

    int getNumTSInfo() {
	return _tsInfoList.Size();
    }

    char* getFileInfo(int i) {
	if ( i < _fileInfoList.Size() )
	    return (char*) _fileInfoList[i];
	else
	    return "";
    }

    int getNumFileInfo() {
	return _fileInfoList.Size();
    }

    char* getLogInfo(int i) {
	if ( i < _logInfoList.Size() )
	    return (char*) _logInfoList[i];
	else
	    return "";
    }

    int getNumLogInfo() {
	return _logInfoList.Size();
    }


    // Admin requests 

    void reqGetTableSetList(ListT<Chain>& tsList) {
    	try
	{
	     _pAdmNet->getTableSetList(tsList);
	}
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqUpdateTableSetInfo(char *tableSet) {
	_tsInfoList.Empty();
	_fileInfoList.Empty();
	_logInfoList.Empty();

	try
	{
		_pAdmNet->getTableSetInfo(Chain(tableSet), _tsInfoList, _fileInfoList, _logInfoList);
	}
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqGetThreadInfo(ListT<Chain>& threadInfo) {

	try
	{
		_pAdmNet->getThreadInfo(threadInfo);
	}
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqGetDbThreadInfo(ListT<Chain>& dbThreadInfo) {
    	
	try
	{
		_pAdmNet->getDbThreadInfo(dbThreadInfo);
	}
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqGetAdmThreadInfo(ListT<Chain>& admThreadInfo) {

    	try
    	{
		_pAdmNet->getAdmThreadInfo(admThreadInfo);
	}
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqGetLogThreadInfo(ListT<Chain>& logThreadInfo) {

    	try
	{
		_pAdmNet->getLogThreadInfo(logThreadInfo);

	}	
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqImportTableSet(char* tableSet, char* impMode, char* impFile) {

    	try
	{

		_pAdmNet->importTableSet(Chain(tableSet), Chain(impMode), Chain(impFile));

	}	
	catch ( Exception e )
	{
	    e.print();
	}

    }

    void reqExportTableSet(char* tableSet, char* expMode, char* expFile) {

        try
	{
    	 	_pAdmNet->exportTableSet(Chain(tableSet), Chain(expMode), Chain(expFile));

	}	
	catch ( Exception e )
	{
	    e.print();
	}

    }


private:
    CegoAdmNet* _pAdmNet;
    ListT<Chain> _tsInfoList;
    ListT<Chain> _fileInfoList;
    ListT<Chain> _logInfoList;

};

MODULE = CegoCtl	PACKAGE = CegoCtl
 
CegoCtlDriver* 
CegoCtlDriver::new()
 
void 
CegoCtlDriver::DESTROY()

void
CegoCtlDriver::connect(server, port, user, pwd)
        char* server;
	int port;
	char* user;
	char* pwd;

	PPCODE:
	       THIS->connect(server,port,user,pwd);

void
CegoCtlDriver::getTableSetList()
	PPCODE:
		ListT<Chain> tsList;
	       	THIS->reqGetTableSetList(tsList);
	       	Chain *pI = tsList.First();
	       	while ( pI )
	       	{
			XPUSHs(sv_2mortal(newSVpv((char*)*pI,0)));
			pI = tsList.Next();
		}

void
CegoCtlDriver::updateTSInfo(tableset)
        char* tableset;
	PPCODE:
	       THIS->reqUpdateTableSetInfo(tableset);

void
CegoCtlDriver::getTSParamList()
	PPCODE:
	       for(int i=0;i<THIS->getNumTSInfo();i++) {
			XPUSHs(sv_2mortal(newSVpv(THIS->getTSInfo(i),0)));		
		}

void
CegoCtlDriver::getTSDataFileList()
	PPCODE:
	       for(int i=0;i<THIS->getNumFileInfo();i++) {
			XPUSHs(sv_2mortal(newSVpv(THIS->getFileInfo(i),0)));		
		}

void
CegoCtlDriver::getTSLogFileList()
	PPCODE:
	       for(int i=0;i<THIS->getNumLogInfo();i++) {
			XPUSHs(sv_2mortal(newSVpv(THIS->getLogInfo(i),0)));		
		}

void
CegoCtlDriver::getThreadInfo()
	PPCODE:
		ListT<Chain> threadInfo;
	       	THIS->reqGetThreadInfo(threadInfo);
	       	Chain *pI = threadInfo.First();
	       	while ( pI )
	       	{
			XPUSHs(sv_2mortal(newSVpv((char*)*pI,0)));
			pI = threadInfo.Next();
		}

void
CegoCtlDriver::getDbThreadInfo()
	PPCODE:
		ListT<Chain> dbThreadInfo;
	       	THIS->reqGetDbThreadInfo(dbThreadInfo);
	       	Chain *pI = dbThreadInfo.First();
	       	while ( pI )
	       	{
			XPUSHs(sv_2mortal(newSVpv((char*)*pI,0)));
			pI = dbThreadInfo.Next();
		}


void
CegoCtlDriver::getAdmThreadInfo()
	PPCODE:
		ListT<Chain> admThreadInfo;
	       	THIS->reqGetAdmThreadInfo(admThreadInfo);
	       	Chain *pI = admThreadInfo.First();
	       	while ( pI )
	       	{
			XPUSHs(sv_2mortal(newSVpv((char*)*pI,0)));
			pI = admThreadInfo.Next();
		}

void
CegoCtlDriver::getLogThreadInfo()
	PPCODE:
		ListT<Chain> logThreadInfo;
	       	THIS->reqGetLogThreadInfo(logThreadInfo);
	       	Chain *pI = logThreadInfo.First();
	       	while ( pI )
	       	{
			XPUSHs(sv_2mortal(newSVpv((char*)*pI,0)));
			pI = logThreadInfo.Next();
		}

void
CegoCtlDriver::importTableSet(tableSet, impMode, impFile)
        char* tableSet;
        char* impMode;
        char* impFile;
	PPCODE:
	       	THIS->reqImportTableSet(tableSet, impMode, impFile);

void
CegoCtlDriver::exportTableSet(tableSet, expMode, expFile)
        char* tableSet;
        char* expMode;
        char* expFile;
	PPCODE:
	       	THIS->reqExportTableSet(tableSet, expMode, expFile);





